package com.adc.da.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.imageio.ImageIO;
import javax.imageio.stream.FileImageOutputStream;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

/**
 * 图片处理工具类
 */
public class PicEdit {

    private static final Logger logger = LoggerFactory.getLogger(PicEdit.class);

    /**
     * 获取图片byte数组
     * @param imgLocalUrl
     * @return
     * @throws IOException
     */
    public static byte[] getByteByPic(String imgLocalUrl) throws IOException {
        File imageFile = new File(imgLocalUrl);
        InputStream inStream = new FileInputStream(imageFile);
        BufferedInputStream bis = new BufferedInputStream(inStream);
        BufferedImage bm = ImageIO.read(bis);
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        //获取文件后缀名
        String type = imgLocalUrl.substring(imgLocalUrl.lastIndexOf('.') + 1);
        ImageIO.write(bm, type, bos);
        bos.flush();
        return bos.toByteArray();
    }

    /**
     * 获取图片数组
     * @param file
     * @param type
     * @return
     * @throws IOException
     */
    public static byte[] getByteByPic(File file, String type) throws IOException {
        InputStream inStream = new FileInputStream(file);
        BufferedInputStream bis = new BufferedInputStream(inStream);
        BufferedImage bm = ImageIO.read(bis);
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        ImageIO.write(bm, type, bos);
        bos.flush();
        return bos.toByteArray();
    }

    /**
     * 将图片压缩到指定大小以内
     *
     * @param srcImgData 源图片数据
     * @param maxSize    目的图片大小
     * @return 压缩后的图片数据
     */
    public static byte[] compressUnderSize(byte[] srcImgData, long maxSize) {
        double scale = 0.9;
        byte[] imgData = Arrays.copyOf(srcImgData, srcImgData.length);

        if (imgData.length > maxSize) {
            do {
                try {
                    imgData = compress(imgData, scale);

                } catch (IOException e) {
                    throw new IllegalStateException("压缩图片过程中出错，请及时联系管理员！", e);
                }

            } while (imgData.length > maxSize);
        }

        return imgData;
    }

    /**
     * 按照 宽高 比例压缩
     *
     * @param srcImgData 待压缩图片输入流
     * @param scale 压缩刻度
     * @return 压缩后图片数据
     * @throws IOException 压缩图片过程中出错
     */
    public static byte[] compress(byte[] srcImgData, double scale) throws IOException {
        BufferedImage bi = ImageIO.read(new ByteArrayInputStream(srcImgData));
        // 源图宽度
        int width = (int) (bi.getWidth() * scale);
        // 源图高度
        int height = (int) (bi.getHeight() * scale);

        Image image = bi.getScaledInstance(width, height, Image.SCALE_SMOOTH);
        BufferedImage tag = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        Graphics g = tag.getGraphics();
        g.setColor(Color.RED);
        // 绘制处理后的图
        g.drawImage(image, 0, 0, null);
        g.dispose();

        ByteArrayOutputStream bOut = new ByteArrayOutputStream();
        ImageIO.write(tag, "JPEG", bOut);

        return bOut.toByteArray();
    }
    /**
     * byte数组到图片
     * @param data
     * @param path
     */
    public static void byte2image(byte[] data, String path) {
        if (data.length < 3 || path.equals("")) {
            return;
        }
        try (FileImageOutputStream imageOutput = new FileImageOutputStream(new File(path))) {
            imageOutput.write(data, 0, data.length);
        } catch (IOException e) {
            logger.error("数组转图片失败");
        }
    }

    /**
     * 功能：获取图片像素
     * @param filePath 图片路径
     **/
    public static Map<String, Integer> getPixel(String filePath){
        File file = new File(filePath);
        BufferedImage bi = null;
        Map<String, Integer> map = new HashMap<>();
        try {
            bi = ImageIO.read(file);
        } catch (Exception e) {
            logger.error("获取图片像素时读取文件失败");
            map.put("width",0);
            map.put("height",0);
        }
        if(bi != null){
            // 像素
            int width = bi.getWidth();
            // 像素
            int height = bi.getHeight();
            map.put("width",width);
            map.put("height",height);
            return map;
        }
        return map;
    }
    /**
     * @param srcPath  源图片路径
     * @param desPath  修改大小后图片路径
     * @param scaleSize 图片的修改比例，目标宽度
     */
    public static void resizeImage(String srcPath, String desPath, int scaleSize) throws IOException {

        File srcFile = new File(srcPath);
        Image srcImg = ImageIO.read(srcFile);
        BufferedImage bi = null;
        try {
            bi = ImageIO.read(srcFile);
        } catch (Exception e) {
            logger.error("修改图片像素时读取文件失败");
        }
        if(bi != null){
            // 像素
            float width = bi.getWidth();
            // 像素
            float height = bi.getHeight();
            float scale=width/scaleSize;
            BufferedImage buffImg = null;
            buffImg = new BufferedImage(scaleSize, (int)(height/scale), BufferedImage.TYPE_INT_RGB);
            //使用TYPE_INT_RGB修改的图片会变色
            buffImg.getGraphics().drawImage(
                    srcImg.getScaledInstance(scaleSize, (int)(height/scale), Image.SCALE_SMOOTH), 0,
                    0, null);
            ImageIO.write(buffImg, "JPEG", new File(desPath));
        }
    }

    /**
     * @param srcPath  源图片路径
     * @param desPath  修改大小后图片路径
     * @param scaleSize 图片的修改比例，目标宽度
     */
    public static void resizeImageHight(String srcPath, String desPath, int scaleSize) throws IOException {

        File srcFile = new File(srcPath);
        Image srcImg = ImageIO.read(srcFile);
        BufferedImage bi = null;
        try {
            bi = ImageIO.read(srcFile);
        } catch (Exception e) {
            logger.error("修改图片像素时读取文件失败");
        }
        if(bi != null) {
            // 像素(宽)
            float width = bi.getWidth();
            // 像素(高)
            float height = bi.getHeight();
            float scale=height/scaleSize;
            BufferedImage buffImg = null;
            buffImg = new BufferedImage((int)(width/scale), scaleSize, BufferedImage.TYPE_INT_RGB);
            //使用TYPE_INT_RGB修改的图片会变色
            buffImg.getGraphics().drawImage(
                    srcImg.getScaledInstance((int)(width/scale), scaleSize, Image.SCALE_SMOOTH), 0,
                    0, null);
            ImageIO.write(buffImg, "JPEG", new File(desPath));
        }
    }

    /**
     * png转jpg
     * @param path
     * @return
     */
    public static String pngToJpg(String path) {
        BufferedImage bufferedImage;
        try {
            File file = new File(path);
            // read image file
            bufferedImage = ImageIO.read(file);
            // create a blank, RGB, same width and height, and a white
            // background
            BufferedImage newBufferedImage = new BufferedImage(
                    bufferedImage.getWidth(), bufferedImage.getHeight(),
                    BufferedImage.TYPE_INT_RGB);
            // TYPE_INT_RGB:创建一个RBG图像，24位深度，成功将32位图转化成24位
            newBufferedImage.createGraphics().drawImage(bufferedImage, 0, 0,
                    Color.WHITE, null);
            int i = path.lastIndexOf('/')+ 1;
            String url = path.substring(0, i);
            String fileName = path.substring(i, path.lastIndexOf('.'));
            // write to jpeg file
            ImageIO.write(newBufferedImage, "jpg", new File(url + fileName +".jpg"));
            if (!new File(path).delete()) {
                logger.error("文件删除失败");
            }
            return url + fileName +".jpg";
        } catch (IOException e) {
            logger.error("图片格式转换失败");
            return "";
        }
    }
}
