package com.adc.da.util;

import com.alibaba.fastjson.JSON;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import javax.net.ssl.SSLException;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Random;

/**
 * 图片识别类
 */
@Component
public class TemplateUtils {
    private String boundaryString = getBoundary();
    /**
     * url
     */
    @Value("${ocr.templateDistinguishUrl}")
    private String templateDistinguishUrl;
    /**
     * apiKey
     */
    @Value("${ocr.api_key}")
    private String apiKey;
    /**
     * apiSecret
     */
    @Value("${ocr.api_secret}")
    private String apiSecret;

    /**
     * 图片识别调用此方法
     * 在face++上建立自定义模板
     * 调用face++的外部接口
     * 图片格式：支持JPEG，JPG，PNG，BMP
     * 图片尺寸：图片最长边不超过4096像素，最短边不少于512像素
     * 图片大小：图片大小不超过4M
     *
     * @param file       图片文件
     * @param templateId 模板id
     * @return
     * @throws IOException
     */
    public Map<String, String> templateDistinguish(File file, String templateId) throws IOException {
        if (file != null && StringUtils.isNotEmpty(templateId)) {
            byte[] buff = getBytesFromFile(file);
            buff = PicEdit.compressUnderSize(buff, 2500*1024L);
            // 返回Base64编码过的字节数组字符串
            String baseFile = Base64.encodeBase64String(Objects.requireNonNull(buff));

            int i = file.getName().lastIndexOf(".");
            String lastName = file.getName().substring(i + 1);
            baseFile = "data:image/" + lastName + ";base64," + baseFile;
            HashMap<String, Object> map = new HashMap<>();
            map.put("api_key", apiKey);
            map.put("api_secret", apiSecret);
            map.put("template_id", templateId);
            map.put("image_base64", baseFile);
            try {
                //调用外部接口
                byte[] allResult = post(templateDistinguishUrl, map);
                //处理返回数据
                if (allResult != null && allResult.length > 0) {
                    String resultJson = new String(allResult);
                    TemplateData templateData = JSON.parseObject(resultJson, TemplateData.class);
                    Map<String, String> resultMap = new HashMap<>();
                    List<TemplateResult> results = templateData.getResult();
                    if (!results.isEmpty()) {
                        //去除json中的[]和""
                        setResultMap(results, resultMap);
                        return resultMap;
                    } else {
                        return null;
                    }
                }
            } catch (Exception e) {
                throw new IOException("出现异常");
            }
            return null;
        }
        return null;
    }

    /**
     * 设置返回map
     * @param results
     * @param resultMap
     */
    private void setResultMap(List<TemplateResult> results, Map<String, String> resultMap) {
        for (int i = 0; i < results.size(); i++) {
            if (results.get(i) != null && !results.get(i).getValue().isEmpty() &&
                    results.get(i).getValue().get("text") != null) {
                String value = results.get(i).getValue().get("text").toString();
                if (StringUtils.isNotEmpty(value)) {
                    if (value.contains("[")) {
                        value = value.replace("[", "");
                    }
                    if (value.contains("]")) {
                        value = value.replace("]", "");
                    }
                    if (value.contains("\"")) {
                        value = value.replace("\"", "");
                    }
                    resultMap.put(results.get(i).getKey(), value);
                }
            }
        }
    }

    private byte[] post(String url, HashMap<String, Object> map) throws IOException {
        HttpURLConnection connection;
        URL url1 = new URL(url);
        connection = (HttpURLConnection) url1.openConnection();
        connection.setDoOutput(true);
        connection.setUseCaches(false);
        connection.setRequestMethod("POST");
        connection.setConnectTimeout(30000);
        connection.setReadTimeout(50000);
        connection.setRequestProperty("accept", "*/*");
        connection.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundaryString);
        connection.setRequestProperty("connection", "Keep-Alive");
        connection.setRequestProperty("user-agent", "Mozilla/4.0 (compatible;MSIE 6.0;Windows NT 5.1;SV1)");
        DataOutputStream dataOutputStream = new DataOutputStream(connection.getOutputStream());
        Iterator iter = map.entrySet().iterator();
        while (iter.hasNext()) {
            Map.Entry<String, Object> entry = (Map.Entry) iter.next();
            String key = entry.getKey();
            Object value = entry.getValue();
            dataOutputStream.writeBytes("--" + boundaryString + "\r\n");
            dataOutputStream.writeBytes("Content-Disposition: form-data; name=\"" + key
                    + "\"\r\n");
            dataOutputStream.writeBytes("\r\n");
            dataOutputStream.writeBytes(value + "\r\n");
        }
        dataOutputStream.writeBytes("--" + boundaryString + "--" + "\r\n");
        dataOutputStream.writeBytes("\r\n");
        dataOutputStream.flush();
        dataOutputStream.close();
        InputStream ins = null;
        int code = connection.getResponseCode();
        try {
            if (code == 200) {
                ins = connection.getInputStream();
            } else {
                return new byte[0];
            }
        } catch (SSLException e) {
            throw new SSLException("SSL异常");
        }
        ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
        byte[] buff = new byte[4096];
        int len;
        while ((len = ins.read(buff)) != -1) {
            byteArrayOutputStream.write(buff, 0, len);
        }
        byte[] bytes = byteArrayOutputStream.toByteArray();
        ins.close();
        return bytes;
    }

    /**
     * 获取Boundary
     * @return
     */
    private static String getBoundary() {
        StringBuilder sb = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < 32; ++i) {

            sb.append("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_-".charAt(random.nextInt("ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789_".length())));
        }
        return sb.toString();
    }

    /**
     * 获取图片数组
     * @param f
     * @return
     * @throws IOException
     */
    private static byte[] getBytesFromFile(File f) throws IOException {
        try (FileInputStream stream = new FileInputStream(f)) {
            ByteArrayOutputStream out = new ByteArrayOutputStream(1000);
            byte[] b = new byte[1000];
            int n;
            while ((n = stream.read(b)) != -1) {
                out.write(b, 0, n);
            }
            return out.toByteArray();
        } catch (IOException e) {
            throw new IOException("文件转换字节数组时出现错误");
        }
    }
}
